library(TSA)
library(fUnitRoots)
library(forecast)
library(plotrix)

Bitcoin <- read.csv("C:/Users/Isaac Owusu/Desktop/Statistical Project Files/Data/Bitcoin.csv", header = TRUE)
BitcoinTestData = read.csv("C:/Users/Isaac Owusu/Desktop/Statistical Project Files/Data/TestData.csv", header = TRUE)
PredNum = 30 #Number of predictions
test.Bitcoin = BitcoinTestData$Close[1:PredNum]


#Creating a time series from closing price of Bitcoin

ts.Bitcoin = ts(data.frame(Bitcoin$Close), start = c(2013),  frequency = 1)

#Original time series plot
plot(ts.Bitcoin, main = "Bitcoin Closing Prices from 4-28-2013")

plot(y = ts.Bitcoin, x = zlag(ts.Bitcoin, d=1))

#Check for stationarity
adfTest(ts.Bitcoin, lags = 1, type = "nc", description = NULL)
adf.test(ts.Bitcoin)

#Results suggest that the time series is not stationary
#Find the log time series to try to eliminate the variance
plot(log(ts.Bitcoin))
acf(log(ts.Bitcoin))
pacf(log(ts.Bitcoin))
eacf(log(ts.Bitcoin))
adfTest(log(ts.Bitcoin), lags = 1, type = "nc", description = NULL)
adf.test(log(ts.Bitcoin))

#Find the log time series to try to eliminate the variance
plot(diff(ts.Bitcoin))
acf(diff(ts.Bitcoin))
pacf(diff(ts.Bitcoin))
eacf(diff(ts.Bitcoin))
adfTest(diff(ts.Bitcoin), lags = 1, type = "nc", description = NULL)
adf.test(diff(ts.Bitcoin))
kpss.test(diff(ts.Bitcoin), null = c("Level")) #Null hyp for this test is that test is stationary. Since p-value is high do no reject


#Differencing log time series 
plot(diff(log(ts.Bitcoin)))

#Testing for stationarity of differenced time series
acf(diff(log(ts.Bitcoin)))
pacf(diff(log(ts.Bitcoin)))
eacf(diff(log(ts.Bitcoin)))
adf.test(diff(log(ts.Bitcoin))) #small p-value: reject null. Null hypothesis for test is that TS is not stationary
kpss.test(diff(log(ts.Bitcoin)), null = c("Level")) #Null hyp for this test is that test is stationary. Since p-value is high do no reject


#ARIMA(5,1,5) to be considered from eacf results

#ARMA Subsets
ARMASub = armasubsets(diff(log(ts.Bitcoin)), nar = 8, nma = 8, y.name = "Test", ar.method = "ols")
plot(ARMASub, main = "Plot of Model Subsets")


#ARI(6,1) with an intercept term to be considered (from ARMA Subset results)

#Fitting Models
Mdl1 = arima(log(ts.Bitcoin), order = c(5,1,5), xreg = NULL, include.mean = TRUE, init = NULL, method = "ML")
Mdl2 = arima(log(ts.Bitcoin), order = c(6,1,0), xreg = NULL, include.mean = TRUE, init = NULL, method = "ML")
Mdl3 = arima(log(ts.Bitcoin), order = c(6,1,6), xreg = NULL, include.mean = TRUE, init = NULL, method = "ML")
Mdl4 = arima(log(ts.Bitcoin), order = c(5,1,7), xreg = NULL, include.mean = TRUE, init = NULL, method = "ML")

Mdl1;Mdl2;Mdl3;Mdl4



#Residual Analysis
tsdiag(Mdl1, gof.lag = 5, omit.initial = F)
tsdiag(Mdl2, gof.lag = 5, omit.initial = F)
tsdiag(Mdl3, gof.lag = 5, omit.initial = F)
tsdiag(Mdl4, gof.lag = 5, omit.initial = F)


#Detecting Outliers
iout2 = detectAO(Mdl1)
iOut=detectIO(Mdl1)
iOut$ind

outM1 = detectAO(Mdl1)
outM2 = detectAO(Mdl2)
outM3 = detectAO(Mdl3)
outM4 = detectAO(Mdl4)

#Test for autocorrelation
Box.test(residuals(Mdl1), lag=10, fitdf=0)
Box.test(residuals(Mdl2), lag=10, fitdf=0)
Box.test(residuals(Mdl3), lag=10, fitdf=0)
Box.test(residuals(Mdl4), lag=10, fitdf=0)

#Shapiro Test to test for Normality
shapiro.test(residuals(Mdl1))
shapiro.test(residuals(Mdl2))
shapiro.test(residuals(Mdl3))
shapiro.test(residuals(Mdl4))


#Normality

qqnorm(residuals(Mdl1),main='ARIMA(5,1,5) QQ Plot')
qqline(residuals(Mdl1),col='red')
qqnorm(residuals(Mdl2),main='ARI(6,1) QQ Plot')
qqline(residuals(Mdl2),col='black')
qqnorm(residuals(Mdl3),main='ARIMA(6,1,6) QQ Plot')
qqline(residuals(Mdl3),col='green')
qqnorm(residuals(Mdl4),main='ARIMA(5,1,7) QQ Plot')
qqline(residuals(Mdl4),col='green')


#acf(residuals(Mdl1))
acf(residuals(Mdl1))
acf(residuals(Mdl2))
acf(residuals(Mdl3))
acf(residuals(Mdl4))


#Test for Independence
Box.test(residuals(Mdl1), lag = 15, type="Ljung-Box",fitdf=10)
#Null hyp: Time series is independent. p-value = .9978 indicates this is independent
Box.test(residuals(Mdl2), lag = 15, type="Ljung-Box",fitdf=6)
#Null hyp: Time series is independent. p-value = .6053 indicates this is independent
Box.test(residuals(Mdl3), lag = 15, type="Ljung-Box",fitdf=6)
Box.test(residuals(Mdl4), lag = 15, type="Ljung-Box",fitdf=6)



#Test for Constant Variance
#op <- par(mfrow = c(2, 2),pty = "s")
plot(seq(1:length(rstandard(Mdl1))),rstandard(Mdl1),ylab='Standardized residuals',main='ARIMA(5,1,5)',type='o')
abline(0,0,col="red",lwd=1)
plot(seq(1:length(rstandard(Mdl2))),rstandard(Mdl2),ylab='Standardized residuals',main='ARI(6,1)',type='o')
abline(0,0,col="red",lwd=1)
plot(seq(1:length(rstandard(Mdl3))),rstandard(Mdl3),ylab='Standardized residuals',main='ARIMA(6,1,6)',type='o')
abline(0,0,col="red",lwd=1)
plot(seq(1:length(rstandard(Mdl4))),rstandard(Mdl4),ylab='Standardized residuals',main='ARIMA(5,1,7)',type='o')
abline(0,0,col="red",lwd=1)
#No particular pattern. Can argue in favor of constant variance

#Test for over-fitting
#ARIMA(6,1,6) - Mdl3
Mdl3a = arima(log(ts.Bitcoin), order = c(6,1,7)) #Method = ML produces error. Investigate why
Mdl3b = arima(log(ts.Bitcoin), order = c(7,1,6))

#ARI(6,1) - Mdl2
Mdl2a = arima(log(ts.Bitcoin), order = c(7,1,0), xreg = NULL, include.mean = TRUE, init = NULL, method = "ML")

#ARIMA(5,1,7) - Mdl4
Mdl4a = arima(log(ts.Bitcoin), order = c(6,1,7)) #Method = ML produces error. Investigate why
Mdl4b = arima(log(ts.Bitcoin), order = c(5,1,8))

#Perform operation (coefficient/s.e.) for newly introduced term.Must be less that 1.96 
Mdl3a #Extra term not significant
Mdl3b #Extra term not significant
Mdl2a #Extra term not significant
Mdl4a #Extra term not significant
Mdl4b #Extra term not significant

#Prediction
#ARIMA(5,1,5) Prediction
Bitcoin_Pred1 = rep(NA,PredNum)
WholeData = c(Bitcoin$Close,test.Bitcoin)
for (i in 1:length(Bitcoin_Pred1)){
  Bitcoin_Train = WholeData[1:(length(Bitcoin$Close)-1+i)]
  Bitcoin_Est = arima(log(Bitcoin_Train), order = c(5,1,5), xreg = NULL, include.mean = TRUE, init = NULL, method = "ML")
  Bitcoin_Pred1[i] = predict(Bitcoin_Est, n.ahead = 1)$pred #investigate whether or not to include $pred
}


plot(ts(diff(log(WholeData))),type = "o", xlim = c(0,1700), ylab = "differenced log of whole data", main = "Forecasting with ARIMA(5,1,5)")
points(ts(diff(Bitcoin_Pred1), start = c((length(Bitcoin$Close)+1)), frequency = 1), col = "red")

plot(ts(diff(log(test.Bitcoin))), ylab = "Differenced log of Prices", main = "Forecasting with ARIMA(5,1,5)")
points(ts(diff(Bitcoin_Pred1), frequency = 1), col = "red")

ARIMA515_Metrics1 = accuracy(diff(Bitcoin_Pred1),diff(log(test.Bitcoin)))


#Computing the accuracy of the model having convertered the model back
ARIMA515_Metrics2 = accuracy(exp(Bitcoin_Pred1),test.Bitcoin)



#ARI(6,1) Prediction
Bitcoin_Pred2 = rep(NA,PredNum)
WholeData = c(Bitcoin$Close,test.Bitcoin)
for (i in 1:length(Bitcoin_Pred2)){
  Bitcoin_Train2 = WholeData[1:(length(Bitcoin$Close)-1+i)]
  Bitcoin_Est2 = arima(log(Bitcoin_Train2), order = c(6,1,0), xreg = NULL, include.mean = TRUE, init = NULL, method = "ML")
  Bitcoin_Pred2[i] = predict(Bitcoin_Est2, n.ahead = 1)$pred #investigate whether or not to include $pred
}

#Plotting training data and test data together
#WholeData = c(Bitcoin$Close,BitcoinTestData$Close)

plot(ts(diff(log(WholeData))),type = "o", xlim = c(0,1700), ylab = "differenced log of whole data", main = "Forecasting with ARI(6,1)")
points(ts(diff(Bitcoin_Pred2), start = c((length(Bitcoin$Close)+1)), frequency = 1), col = "red")

plot(ts(diff(log(test.Bitcoin))), ylab = "differenced log of Prices", main = "Forecasting with ARI(6,1)")
points(ts(diff(Bitcoin_Pred2), frequency = 1), col = "red")

#Calculating the metrics for ARI(6,1)
ARI61_Metrics1 = accuracy(diff(Bitcoin_Pred2),diff(log(test.Bitcoin)))

#Computing the accuracy of the model having convertered the model back
ARI61_Metrics2 = accuracy(exp(Bitcoin_Pred2),test.Bitcoin)


#Prediction
#ARIMA(6,1,6) Prediction
Bitcoin_Pred3 = rep(NA,PredNum)
WholeData = c(Bitcoin$Close,test.Bitcoin)
for (i in 1:length(Bitcoin_Pred1)){
  Bitcoin_Train = WholeData[1:(length(Bitcoin$Close)-1+i)]
  Bitcoin_Est = arima(log(Bitcoin_Train), order = c(6,1,6), xreg = NULL, include.mean = TRUE, init = NULL, method = "ML")
  Bitcoin_Pred3[i] = predict(Bitcoin_Est, n.ahead = 1)$pred #investigate whether or not to include $pred
}


plot(ts(diff(log(WholeData))),type = "o", xlim = c(0,1700), ylab = "differenced log of whole data", main = "Forecasting with ARIMA(6,1,6)")
points(ts(diff(Bitcoin_Pred3), start = c((length(Bitcoin$Close)+1)), frequency = 1), col = "red")

plot(ts(diff(log(test.Bitcoin))), ylab = "Differenced log of Prices", main = "Forecasting with ARIMA(6,1,6)")
points(ts(diff(Bitcoin_Pred3), frequency = 1), col = "red")

#Calculating the metrics for ARIMA(6,1,6) 
ARIMA616_Metrics1 = accuracy(diff(Bitcoin_Pred3),diff(log(test.Bitcoin)))

#Computing the accuracy of the model having convertered the model back
ARIMA616_Metrics2 = accuracy(exp(Bitcoin_Pred3),test.Bitcoin)


#Prediction
#ARIMA(5,1,7) Prediction
Bitcoin_Pred4 = rep(NA,PredNum)
WholeData = c(Bitcoin$Close,test.Bitcoin)
for (i in 1:length(Bitcoin_Pred1)){
  Bitcoin_Train = WholeData[1:(length(Bitcoin$Close)-1+i)]
  Bitcoin_Est = arima(log(Bitcoin_Train), order = c(5,1,7), xreg = NULL, include.mean = TRUE, init = NULL, method = "ML")
  Bitcoin_Pred4[i] = predict(Bitcoin_Est, n.ahead = 1)$pred #investigate whether or not to include $pred
}

#Plotting training data and test data together
plot(ts(diff(log(WholeData))),type = "o", xlim = c(0,1700), ylab = "differenced log of whole data", main = "Forecasting with ARIMA(5,1,7)")
points(ts(diff(Bitcoin_Pred4), start = c((length(Bitcoin$Close)+1)), frequency = 1), col = "red")

plot(ts(diff(log(test.Bitcoin))), ylab = "Differenced log of Prices", main = "Forecasting with ARIMA(5,1,7)")
points(ts(diff(Bitcoin_Pred4), frequency = 1), col = "red")

#Calculating the metrics for ARIMA(5,1,7) 
ARIMA517_Metrics1 = accuracy(diff(Bitcoin_Pred4),diff(log(test.Bitcoin)))

#Computing the accuracy of the model having convertered the model back
ARIMA517_Metrics2 = accuracy(exp(Bitcoin_Pred4),test.Bitcoin)



ARIMA515_Metrics2
ARI61_Metrics2
ARIMA616_Metrics2
ARIMA517_Metrics2







#Based on the metrics, ARI (6,1) is selected

#Confidence Intervals
Std_Error = std.error(exp(Bitcoin_Pred2))
low = exp(Bitcoin_Pred2)-1.96*Std_Error
high = exp(Bitcoin_Pred2)+1.96*Std_Error
Conf_Intervals = data.frame(low,high)

cbind(exp(Bitcoin_Pred2),Conf_Intervals,test.Bitcoin)

#30 day ahead prediction
Bitcoin_Pred22 = rep(NA,PredNum)
WholeData22 = c(Bitcoin$Close,exp(Bitcoin_Pred22))
Bitcoin_Train22 = Bitcoin$Close
for (i in 1:length(Bitcoin_Pred22)){
  #Bitcoin_Train3 = WholeData3[1:(length(Bitcoin$Close)-1+i)]
  Bitcoin_Est22 = arima(log(Bitcoin_Train22), order = c(6,1,0), xreg = NULL, include.mean = TRUE, init = NULL, method = "ML")
  Bitcoin_Pred22[i] = predict(Bitcoin_Est22, n.ahead = 1)$pred #investigate whether or not to include $pred
  Bitcoin_Train22 = c((Bitcoin$Close), exp(Bitcoin_Pred22[1:i]))
}

#Comparing the accuracy of 30-day ahead predictions with 1-day ahead predictions
accuracy(test.Bitcoin, exp(Bitcoin_Pred22))
accuracy(test.Bitcoin, exp(Bitcoin_Pred2))
#1 day ahead predictions are better than 30 day ahead

plot(ts(diff(log(test.Bitcoin))), ylab = "differenced log of whole data", main = "Forecasting with ARI(6,1)")
points(ts(diff(Bitcoin_Pred3), frequency = 1), col = "red")

plot(ts(test.Bitcoin), ylab = "differenced log of whole data", main = "Forecasting with ARI(6,1)")
points(ts(exp(Bitcoin_Pred3), frequency = 1), col = "red")

write.csv(cbind(exp(Bitcoin_Pred44),exp(Bitcoin_Pred22),test.Bitcoin), "C:/Users/Isaac Owusu/Desktop/Statistical Project Files/Data/prediction.csv")
